;; Generated package description from wrap-search.el  -*- no-byte-compile: t -*-
(define-package "wrap-search" "4.16.17.0.20240804.163021" "wrapped, non-incremental search" 'nil :commit "4e913585afc59dffa56d39c4511e31552be4e80e" :authors '(("Emanuel Berg" . "incal@dataswamp.org")) :maintainer '("Emanuel Berg" . "incal@dataswamp.org") :keywords '("matching") :url "https://dataswamp.org/~incal/elpa/wrap-search.el")
